@@ -6,7 +6,7 @@
 # http://www.apache.org/licenses/LICENSE-2.0OA
 #
 # Authors:
-# - Wen Guan, <wen.guan@cern.ch>, 2019 - 2020
+# - Wen Guan, <wen.guan@cern.ch>, 2019 - 2022
 
 
 """
@@ -31,6 +31,8 @@ def create_request(scope=None, name=None, requester=None, request_type=None,
                    username=None, userdn=None, transform_tag=None,
                    status=RequestStatus.New, locking=RequestLocking.Idle, priority=0,
                    lifetime=None, workload_id=None, request_metadata=None,
+                   new_poll_period=1, update_poll_period=10,
+                   new_retries=0, update_retries=0, max_new_retries=3, max_update_retries=0,
                    processing_metadata=None):
     """
     Create a request.
@@ -79,7 +81,15 @@ def create_request(scope=None, name=None, requester=None, request_type=None,
                                  transform_tag=transform_tag, status=status, locking=locking,
                                  priority=priority, workload_id=workload_id,
                                  expired_at=expired_at,
+                                 new_retries=new_retries, update_retries=update_retries,
+                                 max_new_retries=max_new_retries, max_update_retries=max_update_retries,
                                  request_metadata=request_metadata, processing_metadata=processing_metadata)
+    if new_poll_period:
+        new_poll_period = datetime.timedelta(seconds=new_poll_period)
+        new_request.new_poll_period = new_poll_period
+    if update_poll_period:
+        update_poll_period = datetime.timedelta(seconds=update_poll_period)
+        new_request.update_poll_period = update_poll_period
     return new_request
 
 
@@ -88,6 +98,8 @@ def add_request(scope=None, name=None, requester=None, request_type=None,
                 username=None, userdn=None, transform_tag=None,
                 status=RequestStatus.New, locking=RequestLocking.Idle, priority=0,
                 lifetime=None, workload_id=None, request_metadata=None,
+                new_poll_period=1, update_poll_period=10,
+                new_retries=0, update_retries=0, max_new_retries=3, max_update_retries=0,
                 processing_metadata=None, session=None):
     """
     Add a request.
@@ -116,6 +128,10 @@ def add_request(scope=None, name=None, requester=None, request_type=None,
                                      username=username, userdn=userdn,
                                      transform_tag=transform_tag, status=status, locking=locking,
                                      priority=priority, workload_id=workload_id, lifetime=lifetime,
+                                     new_poll_period=new_poll_period,
+                                     update_poll_period=update_poll_period,
+                                     new_retries=new_retries, update_retries=update_retries,
+                                     max_new_retries=max_new_retries, max_update_retries=max_update_retries,
                                      request_metadata=request_metadata, processing_metadata=processing_metadata)
         new_request.save(session=session)
         request_id = new_request.request_id
@@ -205,6 +221,46 @@ def get_request(request_id, to_json=False, session=None):
         raise exceptions.NoObject('request request_id: %s cannot be found: %s' % (request_id, error))
 
 
+@read_session
+def get_request_by_id_status(request_id, status=None, locking=False, session=None):
+    """
+    Get a request or raise a NoObject exception.
+
+    :param request_id: The id of the request.
+    :param status: request status.
+    :param locking: the locking status.
+
+    :param session: The database session in use.
+
+    :raises NoObject: If no request is founded.
+
+    :returns: Request.
+    """
+
+    try:
+        query = session.query(models.Request).with_hint(models.Request, "INDEX(REQUESTS REQUESTS_PK)", 'oracle')\
+                                             .filter(models.Request.request_id == request_id)
+
+        if status:
+            if not isinstance(status, (list, tuple)):
+                status = [status]
+            if len(status) == 1:
+                status = [status[0], status[0]]
+            query = query.filter(models.Request.status.in_(status))
+
+        if locking:
+            query = query.filter(models.Request.locking == RequestLocking.Idle)
+            query = query.with_for_update(skip_locked=True)
+
+        ret = query.first()
+        if not ret:
+            return None
+        else:
+            return ret.to_dict()
+    except sqlalchemy.orm.exc.NoResultFound as error:
+        raise exceptions.NoObject('request request_id: %s cannot be found: %s' % (request_id, error))
+
+
 @read_session
 def get_requests(request_id=None, workload_id=None, with_detail=False, with_metadata=False,
                  with_request=False, with_transform=False, with_processing=False, to_json=False, session=None):
@@ -646,7 +702,7 @@ def get_requests_by_requester(scope, name, requester, to_json=False, session=Non
 @transactional_session
 def get_requests_by_status_type(status, request_type=None, time_period=None, request_ids=[], locking=False,
                                 locking_for_update=False, bulk_size=None, to_json=False, by_substatus=False,
-                                only_return_id=False, session=None):
+                                new_poll=False, update_poll=False, only_return_id=False, session=None):
     """
     Get requests.
 
@@ -680,12 +736,13 @@ def get_requests_by_status_type(status, request_type=None, time_period=None, req
                 query = query.filter(models.Request.substatus.in_(status))
             else:
                 query = query.filter(models.Request.status.in_(status))
-            query = query.filter(models.Request.next_poll_at <= datetime.datetime.utcnow())
+        if new_poll:
+            query = query.filter(models.Request.updated_at + models.Request.new_poll_period <= datetime.datetime.utcnow())
+        if update_poll:
+            query = query.filter(models.Request.updated_at + models.Request.update_poll_period <= datetime.datetime.utcnow())
 
         if request_type is not None:
             query = query.filter(models.Request.request_type == request_type)
-        # if time_period is not None:
-        #     query = query.filter(models.Request.updated_at < datetime.datetime.utcnow() - datetime.timedelta(seconds=time_period))
         if request_ids:
             query = query.filter(models.Request.request_id.in_(request_ids))
         if locking:
@@ -696,6 +753,7 @@ def get_requests_by_status_type(status, request_type=None, time_period=None, req
         else:
             query = query.order_by(asc(models.Request.updated_at))\
                          .order_by(desc(models.Request.priority))
+
         if bulk_size:
             query = query.limit(bulk_size)
 
@@ -731,6 +789,11 @@ def update_request(request_id, parameters, session=None):
     try:
         parameters['updated_at'] = datetime.datetime.utcnow()
 
+        if 'new_poll_period' in parameters and type(parameters['new_poll_period']) not in [datetime.timedelta]:
+            parameters['new_poll_period'] = datetime.timedelta(seconds=parameters['new_poll_period'])
+        if 'update_poll_period' in parameters and type(parameters['update_poll_period']) not in [datetime.timedelta]:
+            parameters['update_poll_period'] = datetime.timedelta(seconds=parameters['update_poll_period'])
+
         if 'request_metadata' in parameters and 'workflow' in parameters['request_metadata']:
             workflow = parameters['request_metadata']['workflow']
 
